#!/usr/bin/env python
#-*-coding:utf-8-*-

# "THE WISKEY-WARE LICENSE" (Revision 42):
# <jbc.develop@gmail.com> wrote this file. As long as you retain this notice you
# can do whatever you want with this stuff. If we meet some day, and you think
# this stuff is worth it, you can buy me a wiskey in return Juan BC


#===============================================================================
# DOCS
#===============================================================================

"""Infopython cache framework

"""

#===============================================================================
# IMPORTS
#===============================================================================

import functools
import inspect
import warnings

from _core_cache import AbstractCache


#===============================================================================
# DECORATOR
#===============================================================================

 
def cached(cache):
    """Decorator (or not) for create a cached callable.
    
    You can use as a decorator:
    
    ::
        cache_engine = CacheEngine()
        
        @cached(cache_engine)
        def some_callable(*args, **kwargs):
            pass
    
    or as function
    
    ::
        cache_engine = CacheEngine()
        some_callable = cached(cache_engine)(some_callable)
       
       
    *params*
        
        :cache:
            An instance of an cache engine.
            
        
    """

    assert isinstance(cache, AbstractCache)
    
    def decorator(c):
        
        if not hasattr(c, "__call__"):
            msg = "{} Mut be a callable".format(repr(c))
            raise TypeError(msg)
        
        hashprefix = "_".join((
                getattr(c, "__name__", "") , str(hash(c))
            ))
        
        is_method = inspect.ismethod(c)
            
        @functools.wraps(c)
        def wrap(*args, **kwargs):
            
            # set the base values for hash and data
            key = None
            data = None
            
            # try to create a hash key for this arguments
            try:
                key = "_".join((
                    hashprefix,
                    str(hash(args)),
                    str(hash(tuple(kwargs.items()))),
                ))
            except TypeError:
                # inform if some argument are not hasheable
                warnings.warn("Uncacheable arguments", RuntimeWarning)
                
            else:
                # if has a hash try to recover data from cache
                data = cache.get(key)
                
            # if data still not seted try to make a call
            if data == None:
                if is_method:
                    data = c(args[0], *args[1:], **kwargs)
                else:
                    data = c(*args, **kwargs)
                    
            # if this arguments are cacheable set the data to the cache
            if key:
                cache[key] = data
            return data
            
        return wrap
        
    return decorator

#===============================================================================
# MAIN
#===============================================================================

if __name__ == "__main__":
    print __doc__

