#!/usr/bin/env python
#-*-coding:utf-8-*-

# "THE WISKEY-WARE LICENSE" (Revision 42):
# <jbc.develop@gmail.com> wrote this file. As long as you retain this notice you
# can do whatever you want with this stuff. If we meet some day, and you think
# this stuff is worth it, you can buy me a wiskey in return Juan BC


#===============================================================================
# DOCS
#===============================================================================

"""Abstraction layer of various online services for retrieves informations abour 
isources"""


#===============================================================================
# IMPORTS
#===============================================================================

import abc
import random


#===============================================================================
# ABSTRACT CLASS and exception
#===============================================================================

class AbstractCache(object):
    """Base classes for create caches in infopython"""
    
    __metaclass__ = abc.ABCMeta
    
    def __init__(self, ttl=60):
        """Creates a new instance
        
        params
            ``ttl``
                Time to live of the data.
        """
        self._ttl = ttl
    
    def __getitem__(self, k):
        self._try_clear_cache()
        return self.retrieve_data(k)
        
    def __setitem__(self, k, v):
        self._try_clear_cache()
        self.store_data(k, self.ttl, v)
    
    def _try_clear_cache(self):
        if self._ttl != None and random.randint(0, 100) < 10:
            self.clear_expired()
    
    def get(self, k, d=None):
        """C.get(k[,d]) -> C.pull(k) if k in C and can be retrieved, else d.  
        d defaults to None.

        """
        try:
            return self[k]
        except Exception:
            return d
    
    @property
    def ttl(self):
        self._try_clear_cache()
        return self._ttl

    @ttl.setter
    def ttl(self, ttl):
        self._try_clear_cache()
        self._ttl = ttl
    
    #===========================================================================
    # ABSTRACTS METHODS
    #===========================================================================
       
    @abc.abstractmethod
    def store_data(self, k, ttl, v):
        """Set the data and the ttl in to cache.
        
        """
        raise NotImplementedError()
            
    @abc.abstractmethod
    def retrieve_data(self, k):
        """Retrieve a data from a cache.
        
        """
        raise NotImplementedError()
        
    @abc.abstractmethod
    def clear_expired(self):
        """Deletes all expired data from the cache
        
        """
        raise NotImplementedError()
        
    @abc.abstractmethod
    def clear(self):
        """Removes all data from cache
        
        """
        raise NotImplementedError()
        
                
#===============================================================================
# MAIN
#===============================================================================

if __name__ == "__main__":
    print __doc__

