#!/usr/bin/env python
#-*-coding:utf-8-*-

# "THE WISKEY-WARE LICENSE" (Revision 42):
# <jbc.develop@gmail.com> wrote this file. As long as you retain this notice you
# can do whatever you want with this stuff. If we meet some day, and you think
# this stuff is worth it, you can buy me a wiskey in return Juan BC


#===============================================================================
# DOCS
#===============================================================================

"""Implementation of inmemory cache with a dict"""


#===============================================================================
# IMPORTS
#===============================================================================

import collections
import time

import _core_cache

#===============================================================================
# DICT CACHE CLASS
#===============================================================================

class DictCache(_core_cache.AbstractCache):
    """Implementation of a cache in a memory dictionary
    
    """
    def __init__(self, ttl=60):
        """Creates a new instance
        
        params:
            
            ``ttl``
                Time to live of the data.
                
        """
        super(DictCache, self).__init__(ttl=ttl)
        self._ich = collections.OrderedDict()
        self._ttds = collections.OrderedDict()
   
    def store_data(self, k, ttl, v):
        self._ich[k] = v
        self._ttds[k] = (
            time.time() + ttl if ttl != None else None
        )
        
    def retrieve_data(self, k):
        ttd = self._ttds.get(k, 0)
        if ttd == None or time.time() < ttd:
            return self._ich[k]
        elif ttd:
            self._ttds.pop(k)
            self._ich.pop(k)
    
    def clear_expired(self):
        for k, ttd in self._ttds.items():
            if ttd != None and ttd < time.time():
                self._ttds.pop(k)
                self._ich.pop(k)
            else:
                break
    
    def clear(self):
        self._ich.clear()
        self._ttds.clear()

        
#===============================================================================
# MAIN
#===============================================================================

if __name__ == "__main__":
    print __doc__

