#!/usr/bin/env python
#-*-coding:utf-8-*-

# "THE WISKEY-WARE LICENSE" (Revision 42):
# <jbc.develop@gmail.com> wrote this file. As long as you retain this notice you
# can do whatever you want with this stuff. If we meet some day, and you think
# this stuff is worth it, you can buy me a wiskey in return Juan BC

# Thanks to: Juan Manuel Garcia <jmg.utn@gmail.com>

#===============================================================================
# DOCS
#===============================================================================

"""Implementation of memcached for infopython"""


#===============================================================================
# IMPORTS
#===============================================================================

import time
import hashlib
import pickle

import memcache

import _core_cache


#===============================================================================
# DICT CACHE CLASS
#===============================================================================
        
class MemcachedCache(_core_cache.AbstractCache):
    """Implementation of a memcache for infopython
    
    """
    def __init__(self, hosts, debug=0, ttl=60):
        """Creates a new instance
        
        params:
            
            ``hosts``
                A list of *hosts:port* of memcache daemons.
            
            ``debug``
                Debug levels of memcache.
                
            ``ttl``
                Time to live of the data.
                
        """
        super(MemcachedCache, self).__init__(ttl=ttl)
        self._mc = memcache.Client(hosts, debug=debug)
        self._prefix = hashlib.sha1(
            str(time.time()) + str(id(self))
        ).hexdigest() + "_"
   
    def ping(self):
        """Return *True* if any server is response
        
        """
        try:
            return bool(self._mc.get_stats())
        except:
            return False
   
    def store_data(self, k, ttl, v):
        if self.ping():
            key = self._prefix + k
            value = pickle.dumps(v)
            ttl = ttl if ttl != None else 0
            self._mc.set(key, value, time=ttl)
        
    def retrieve_data(self, k):
        if self.ping():
            key = self._prefix + k
            value = self._mc.get(key)
            if value:
                return pickle.loads(value)
        return None
    
    def clear_expired(self):
        pass
    
    def clear(self):
        if self.ping():
            self._mc.flush_all()
            

#===============================================================================
# MAIN
#===============================================================================

if __name__ == "__main__":
    print __doc__

