#!/usr/bin/env python
#-*-coding:utf-8-*-

# "THE WISKEY-WARE LICENSE" (Revision 42):
# <jbc.develop@gmail.com> wrote this file. As long as you retain this notice you
# can do whatever you want with this stuff. If we meet some day, and you think
# this stuff is worth it, you can buy me a wiskey in return Juan BC

# Slightly inspired by:
#    http://code.prashanthellina.com/code/get_alexa_rank.py

#===============================================================================
# DOCS
#===============================================================================

"""A simple Alexa (http://www.alexa.com/) python Api."""


#===============================================================================
# IMPORTS
#===============================================================================

import urllib2
from xml.dom import minidom
import datetime


#===============================================================================
# CONSTANTS
#===============================================================================

_ALEXA_URL = "http://data.alexa.com/data?cli=10&dat=snbamz&url="


#===============================================================================
# FUNCTIONS
#===============================================================================

def rank(hostname):
        """Retrieves data from Alexa
        
        params:
            
            ::hostname::
                host name on which we want to know the alexa rank.
                
        ::return::
        
            dict with the data.
        
        """
        
        rank = {u"host": unicode(hostname)}
        stream = urllib2.urlopen(_ALEXA_URL + hostname)
        data = minidom.parse(stream)
        
        sd = data.getElementsByTagName("SD") or None
        rank[u"site_type"] = sd[0].getAttribute("TITLE") if sd else None
        rank[u"flags"] = sd[0].getAttribute("FLAGS") if sd else None
        
        popularity = data.getElementsByTagName("POPULARITY") or None
        if popularity:
            popularity = float(popularity[0].getAttribute("TEXT"))
        rank[u"popularity"] = popularity
        
        linksin = data.getElementsByTagName("LINKSIN") or None
        if linksin:
            linksin = float(linksin[0].getAttribute("NUM"))
        rank[u"linksin"] = linksin
        
        reach = data.getElementsByTagName("REACH") or None
        if reach:
            reach = float(reach[0].getAttribute("RANK"))
        rank[u"reach"] = reach
        
        rank_delta = data.getElementsByTagName("RANK") or None
        if rank_delta:
            rank_delta = float(rank_delta[0].getAttribute("DELTA"))
        rank[u"rank"] = rank_delta
        
        rls = data.getElementsByTagName("RLS")
        rank[u"relateds"] = {u"more": float(rls[0].getAttribute("more")) 
                                    if rls else None,
                            u"relateds": []}
        for rd in data.getElementsByTagName("RL"):
            r = {}
            r[u"href"] = rd.getAttribute("HREF")
            r[u"title"] = rd.getAttribute("TITLE")
            rank[u"relateds"][u"relateds"].append(r)
        
        rank[u"assocs"] = []
        for ad in data.getElementsByTagName("ASSOC"):
            rank[u"assocs"].append(ad.getAttribute("ID"))
            
        title = data.getElementsByTagName("TITLE")
        rank[u"title"] = title[0].getAttribute("TEXT") if title else None
        
        addr = data.getElementsByTagName("ADDR")
        rank[u"addr"] = {u"street": addr[0].getAttribute("STREET")
                                    if addr else None,
                        u"city": addr[0].getAttribute("CITY") if addr else None,
                        u"state": addr[0].getAttribute("STATE")
                                  if addr else None,
                        u"zip": addr[0].getAttribute("ZIP") if addr else None,
                        u"country": addr[0].getAttribute("COUNTRY") 
                                    if addr else None, }

        created = data.getElementsByTagName("CREATED") or None
        if created:
            created = created[0]
            d = int(created.getAttribute("DAY"))
            m = int(created.getAttribute("MONTH"))
            y = int(created.getAttribute("YEAR"))
            created = datetime.datetime(y, m, d)
        rank[u"created"] = created
        
        phone = data.getElementsByTagName("PHONE") or None
        if phone:
            phone = phone[0].getAttribute("NUMBER")
        rank[u"phone"] = phone
        
        owner = data.getElementsByTagName("OWNER") or None
        if owner:
            owner = owner[0].getAttribute("NAME")
        rank[u"owner"] = owner
        
        email = data.getElementsByTagName("EMAIL") or None
        if email:
            email = email[0].getAttribute("ADDR")
        rank[u"email"] = email
        
        lang = data.getElementsByTagName("LANG") or None
        if lang:
            lang = lang[0].getAttribute("LEX")
        rank[u"lang"] = lang
        
        speed = data.getElementsByTagName("SPEED") or None
        rank[u"speed"] = {u"avg": float(speed[0].getAttribute("TEXT")) / 10 
                                if speed else None,
                          u"percent_sites_slower": float(speed[0].getAttribute("PCT")) 
                                                 if speed else None, }

        revs = data.getElementsByTagName("REVIEWS") or None
        rank[u"reviews"] = {u"avg": float(revs[0].getAttribute("AVG")) 
                                    if revs else None,
                            u"total": float(revs[0].getAttribute("NUM") or 0.0) 
                                    if revs else None}
        
        child = data.getElementsByTagName("CHILD") or None
        if child:
            child = float(child[0].getAttribute("SRATING"))
        rank[u"child"] = child
        
        keywords = []
        for k in data.getElementsByTagName("KEYWORD"):
            keywords.append(k.getAttribute("VAL"))
        rank[u"keywords"] = keywords
        
        site = data.getElementsByTagName("SITE") or None
        if site:
            site = {u"base": site[0].getAttribute("BASE"),
                    u"title": site[0].getAttribute("TITLE"),
                    u"desc": site[0].getAttribute("DESC")}
        rank[u"site"] = site    
        
        categories = []
        for c in data.getElementsByTagName("CAT"):
            categories.append({u"id": c.getAttribute("ID"),
                               u"title": unicode(c.getAttribute("TITLE")),
                               u"cid": int(c.getAttribute("CID"))})
        rank[u"categories"] = categories
        
        return rank
    

#===============================================================================
# MAIN
#===============================================================================

if __name__ == '__main__':
    print __doc__



