#!/usr/bin/env python
#-*-coding:utf-8-*-

# "THE WISKEY-WARE LICENSE" (Revision 42):
# <jbc.develop@gmail.com> wrote this file. As long as you retain this notice you
# can do whatever you want with this stuff. If we meet some day, and you think
# this stuff is worth it, you can buy me a wiskey in return Juan BC


#===============================================================================
# DOCS
#===============================================================================

"""Retrieves HostIP geo positional data

More Info:
    http://www.hostip.info/
    
"""


#===============================================================================
# IMPORTS
#===============================================================================

import urllib2
import socket
from xml.dom import minidom


#===============================================================================
# CONSTANTS
#===============================================================================

_HOSTIP_URL = "http://api.hostip.info/?ip={ip}"


#===============================================================================
# CLASS
#===============================================================================

def location_by_ip(ip):
    """Retrieves data from location data of the giving ip.
    
    params:
        
        ::ip::
            ip on which we want to know the location.
            
    ::return::
    
        dict with the data.
    
    """
    
    info = {}
    url = _HOSTIP_URL.format(ip=ip)
    stream = urllib2.urlopen(url)
    data = minidom.parse(stream)
    
    info[u"ip"] = unicode(ip)
    
    desc = data.getElementsByTagName("gml:description")[0].firstChild
    info[u"description"] = desc.data

    name = data.getElementsByTagName("gml:name")[0].firstChild
    info[u"name"] = name.data

    location = data.getElementsByTagName("Hostip")[0]

    location_name = location.getElementsByTagName("gml:name")[0].firstChild
    info[u"location_name"] = location_name.data
    
    cntry = location.getElementsByTagName("countryName")[0].firstChild
    info[u"contry"] = cntry.data
    
    cntry_a = location.getElementsByTagName("countryAbbrev")[0].firstChild
    info[u"contry_abbrev"] = cntry_a.data
    
    crd = location.getElementsByTagName("gml:coordinates")
    if crd:
        crd = crd[0].firstChild
        crd = tuple(map(float, crd.data.split(",", 1)))
    else:
        crd = (None, None)
    info[u"coordinates"] = crd
    
    return info


def location_by_hostname(hostname):
        """Retrieves data from location data resolving all the ip's from 
        a hostname.
        
        params:
            
            ``hostname``
                host name on which we want to know the location.
                
        return:
        
            dict with the data.
        
        """
        
        infos = {u"host": unicode(hostname),
                 u"ips": {}}
                 
        for ip in socket.gethostbyname_ex(hostname)[2]:
            infos["ips"][ip] = location_by_ip(ip)
        
        return infos


#===============================================================================
# MAIN
#===============================================================================

if __name__ == "__main__":
    print location_by_hostname("google.com")
    print __doc__
