#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# THE WISKEY-WARE LICENSE
# -----------------------
#
# "THE WISKEY-WARE LICENSE":
# <jbc dot develop at gmail dot com> wrote this file. As long as you retain this notice you
# can do whatever you want with this stuff. If we meet some day, and you think
# this stuff is worth it, you can buy me a wiskey in return juan
#

#===============================================================================
# DOC
#===============================================================================

"""This module contains the abstract class which serves as base class for all 
definitions of mass-media types.

"""


#===============================================================================
# CLASS
#===============================================================================

class Media(object):
    """This class class which serves as base class for all definitions of 
    mass-media types.

    The derivated class must redefine the method ``get_name`` that returns a 
    unique name for each instance.
    
    """
    
    def __init__(self, name):
        assert isinstance(name, basestring)
        self._name = name

    def __eq__(self, obj):
        """x.__eq__(y) <==> x==y"""
        return obj != None \
            and isinstance(obj, self.__class__) \
            and obj.name == self.name

    def __hash__(self):
        """x.__hash__() <==> hash(x)"""
        return hash(self.name)
    
    def __str__(self):
        """x.__str__() <==> str(x)"""
        return repr(self)
    
    def __repr__(self):
        """x.__repr__() <==> repr(x)""" 
        msg = "<{classname} (name={name}) at {id}>"
        return  msg.format(classname=self.__class__.__name__,
                            name=str(self.name),
                            id=hex(id(self)))
    
    @property
    def name(self):
        return self._name

#===============================================================================
# MAIN
#===============================================================================

if __name__ == "__main__":
    print(__doc__)
