#!/usr/bin/env python
#-*-coding:utf-8-*-

# "THE WISKEY-WARE LICENSE" (Revision 42):
# <jbc.develop@gmail.com> wrote this file. As long as you retain this notice you
# can do whatever you want with this stuff. If we meet some day, and you think
# this stuff is worth it, you can buy me a wiskey in return Juan BC


#===============================================================================
# DOCS
#===============================================================================

"""A classes for intifiying theories"""


#===============================================================================
# IMPORT
#===============================================================================

import media
import datetime

from util import dttools

#===============================================================================
# CLASSES
#===============================================================================

class Theory(object):

    def __init__(self, name, mtype, medias):
        assert isinstance(name, basestring)
        assert issubclass(mtype, media.Media)
        assert hasattr(medias, "__iter__")
        self._sorted_at = None
        self._name = name
        self._mtype = mtype
        self._medias = []
        for m in medias:
            self.add(m)

    def __contains__(self, m):
        return m in self._medias
    
    def __getitem__(self, i):
        return self._medias[i]

    def __iter__(self):
        return iter(self._medias)

    def __str__(self):
        return repr(self)

    def __repr__(self):
        return "<{} '{}' (MType: {}) at {}>".format(
            self.__class__.__name__,
            self._name,
            self._mtype.__name__,
            hex(id(self))
        )

    def __len__(self):
        return len(self._medias)

    def add(self, m):
        assert isinstance(m, self._mtype)
        if m not in self._medias:
            self._medias.append(m)
            return True
        return False

    def sort(self, key, reverse=False):
        self._medias.sort(key=key, reverse=reverse)
        self._sorted_at = datetime.datetime.now()

    def clear(self):
        self._medias = []

    def remove(self, m):
        self._medias.remove(m)
        
    def index(self, m):
        return self._medias.index(m)
        
    def create_media(self, *media_args, **media_kwargs):
        m = self._mtype(*media_args, **media_kwargs)
        return self.add(m)
        
    def remove_media(self, name):
        for m in self._medias:
            if m.name == name:
                self.remove(m)
                return True
        return False

    def dict_resume(self):
        return {
            "name": self._name,
            "type": type(self).__name__,
            "media_type": self._mtype.__name__,
            "datetime": dttools.strftime(datetime.datetime.now()),
            "medias": tuple([m.name for m in self._medias]),
            "sorted_at": dttools.strftime(self._sorted_at) if self._sorted_at else None
        }

    @property
    def sorted_at(self):
        return self._sorted_at

    @property
    def name(self):
        return self._name

    @property
    def media_type(self):
        return self._mtype


#===============================================================================
# MAIN
#===============================================================================

if __name__ == "__main__":
    print __doc__
