#!/usr/bin/env python
#-*-coding:utf-8-*-

# "THE WISKEY-WARE LICENSE" (Revision 42):
# <jbc.develop@gmail.com> wrote this file. As long as you retain this notice you
# can do whatever you want with this stuff. If we meet some day, and you think
# this stuff is worth it, you can buy me a wiskey in return Juan BC


#===============================================================================
# DOC
#===============================================================================

"""Tools for manipulate strings"""


#===============================================================================
# IMPORTS
#===============================================================================

import re
import unicodedata


#===============================================================================
# CONSTANTS
#===============================================================================

RX_END_WORD = re.compile("\W+$", re.UNICODE)
RX_START_END_WORD = re.compile("^\W+|\W+$", re.UNICODE)
RX_NON_ALPHA = re.compile("\W*", re.UNICODE)
RX_SPLITTER = re.compile("[^a-zA-Z0-9]*", re.UNICODE)

RX_A = re.compile("á|à|ä|â|Á|À|Ä|Â", re.UNICODE)
RX_I = re.compile("í|ì|ï|î|Í|Ì|Ï|Î", re.UNICODE)
RX_U = re.compile("ú|ù|ü|û|Ú|Ù|Ü|Û", re.UNICODE)
RX_E = re.compile("é|è|ë|ê|É|È|Ë|Ê", re.UNICODE)
RX_O = re.compile("ó|ò|ô|ö|Ó|Ò|Ô|Ö", re.UNICODE)
RX_N = re.compile("ñ|Ñ", re.UNICODE)


#===============================================================================
# FUNCTIONS
#===============================================================================

def normalize(string):
    """Convert a str or unicode into a lowercase ascii str.
    
    Hints:
        - Removes all accents.
        - Remove leading and trailing whitespace.
        - Convert the string to lowercase.
        - Remove leading and trailing non alphanumeric chars.
        - Convert "Ñ or ñ" to "n".
        
    Example:
        
    ::
        >>> normalize(u"ñAtò")
        nato

    """
    string = string.lower()
    string = string.strip()
    string = RX_A.sub("a", string)
    string = RX_I.sub("i", string)
    string = RX_U.sub("u", string)
    string = RX_E.sub("e", string)
    string = RX_O.sub("o", string)
    string = RX_N.sub("n", string)
    string = RX_START_END_WORD.sub("", string)
    string = string if isinstance(string, unicode) else unicode(string)
    return unicodedata.normalize('NFKD', string).encode('ASCII', 'ignore').lower()


def split_non_alpha(string):
    """Split a given unicode text by non alpha characters
    
    Example:
    
    ::
        >>> split(u"foo$fuu%fii")
        [u"foo", u"fuu", i"fii"]
    
    """
    return RX_SPLITTER.split(string)


#===============================================================================
# MAIN
#===============================================================================

if __name__ == "__main__":
    print __doc__
