#!/usr/bin/env python
#-*-coding:utf-8-*-

# "THE WISKEY-WARE LICENSE" (Revision 42):
# <jbc.develop@gmail.com> wrote this file. As long as you retain this notice you
# can do whatever you want with this stuff. If we meet some day, and you think
# this stuff is worth it, you can buy me a wiskey in return Juan BC


#===============================================================================
# DOC
#===============================================================================

"""Tools for manipulate strings"""


#===============================================================================
# IMPORTS
#===============================================================================

import datetime

try:
    import cStringIO as StringIO
except ImportError:
    import StringIO

from xml.etree import ElementTree

import dttools


#===============================================================================
# CLASS
#===============================================================================

class XMLParser(object):
    """Convert settings.py bind variables to XML format"""

    def load(self, stream):

        def load_dict(de):
            d = {}
            for ie in de.getchildren():
                k = ie.attrib["name"]
                v = dispatch(ie)[1]
                d[k] = v
            return de.tag, d

        def load_iter(le):
            l = []
            for ie in le.getchildren():
                l.append(dispatch(ie)[1])
            if le.attrib["type"] == "tuple":
                l = tuple(l)
            return le.tag, l

        def load_attr(ae):
            switch = {
                type(None).__name__: lambda x: None,
                int.__name__: int,
                float.__name__: float,
                bool.__name__: lambda x: True if x.lower() == "true" else False,
                str.__name__: str,
                unicode.__name__: unicode,
                datetime.datetime.__name__: dttools.strptime,
            }
            cast = switch.get(ae.attrib["type"], error)
            return ae.tag, cast(ae.text)

        def dispatch(elem):
            switch = {
                type(None).__name__: load_attr,
                int.__name__: load_attr,
                float.__name__: load_attr,
                bool.__name__: load_attr,
                str.__name__: load_attr,
                unicode.__name__: load_attr,
                list.__name__: load_iter,
                tuple.__name__: load_iter,
                dict.__name__: load_dict,
                datetime.datetime.__name__: load_attr
            }
            func = switch.get(elem.attrib["type"], error)
            return func(elem)

        def error(obj):
            msg = "{name} invalid type '{type}'".format(name=str(obj), type=obj.attrib["type"])
            raise TypeError(msg)

        tree = ElementTree.parse(stream)
        return dispatch(tree.getroot())[1]

    def dump(self, stream, obj, rootname="xml"):

        def dump_dict(name, d):
            d_elem = ElementTree.Element(name, attrib={"type": "dict"})
            for k, v in d.items():
                item_elem = dispatch("item", v)
                item_elem.attrib["name"] = k
                d_elem.append(item_elem)
            return d_elem

        def dump_iter(name, l):
            l_elem = ElementTree.Element(name, attrib={"type": type(l).__name__})
            for v in l:
                l_elem.append(dispatch("item", v))
            return l_elem

        def dump_attr(name, a):
            a_elem = ElementTree.Element(name, attrib={"type": type(a).__name__})
            a_elem.text = "" if a == None else str(a)
            return a_elem

        def dump_datetime(name, dt):
            dt_elem = ElementTree.Element(name, attrib={"type": type(dt).__name__})
            dt_elem.text = dttools.strftime(dt)
            return dt_elem

        def error(name, obj):
            msg = "{name} invalid type '{type}'".format(name=name, type=type(obj))
            raise TypeError(msg)

        def dispatch(name, obj):
            switch = {
                type(None).__name__: dump_attr,
                int.__name__: dump_attr,
                float.__name__: dump_attr,
                bool.__name__: dump_attr,
                str.__name__: dump_attr,
                unicode.__name__: dump_attr,
                list.__name__: dump_iter,
                tuple.__name__: dump_iter,
                dict.__name__: dump_dict,
                datetime.datetime.__name__: dump_datetime
            }
            func = switch.get(type(obj).__name__, error)
            return func(name, obj)

        
        root_elem = dispatch(rootname, obj)
        
        tree = ElementTree.ElementTree(root_elem)
        tree.write(stream, encoding="utf-8")


#===============================================================================
# FUNCTIONS
#===============================================================================

def load(fp):
    """Load a fp"""
    parser = XMLParser()
    return parser.load(fp)


def loads(code):
    fp = StringIO.StringIO(code)
    return load(fp)


def dump(fp, obj, rootname="xml"):
    """Dump a object and header into fp.
    
    """
    parser = XMLParser()
    parser.dump(fp, obj, rootname)


def dumps(obj, rootname="xml"):
    fp = StringIO.StringIO()
    dump(fp, obj, rootname)
    return fp.getvalue()


#===============================================================================
# MAIN
#===============================================================================

if __name__ == "__main__":
    print __doc__
