#!/usr/bin/env python
# -*- coding: utf-8 -*-

# "THE WISKEY-WARE LICENSE":
# <jbc.develop@gmail.com> wrote this file. As long as you retain this notice you
# can do whatever you want with this stuff. If we meet some day, and you think
# this stuff is worth it, you can buy me a WISKEY in return Juan BC

#===============================================================================
# DOC
#===============================================================================

"""Test for infopython.cache

All test of cache are tested with the decorator ``cached``

"""

#===============================================================================
# IMPORTS
#===============================================================================

import unittest
import random
import time
import sha

from infopython import cache
from infopython.cache import memcached_cache, dict_cache


#===============================================================================
# CONSTANS
#===============================================================================

# all ttl of cache test 
TTL = 1.1


#===============================================================================
# TEST
#===============================================================================

class BaseCacheTest(object):
    
    def test_basic_gears(self):
        
        if hasattr(self, "skip_reason") and self.skip_reason != None:
            self.skipTest(self.skip_reason)
        
        buff = set()
        
        @cache.cached(self.engine)
        def cached_func0(*args, **kwargs):
            new_data = sha.sha(str(random.random())).hexdigest()
            while new_data in buff:
                new_data = sha.sha(str(random.random())).hexdigest()
            return new_data
        
        @cache.cached(self.eternal)
        def cached_func1(*args, **kwargs):
            new_data = sha.sha(str(random.random())).hexdigest()
            while new_data in buff:
                new_data = sha.sha(str(random.random())).hexdigest()
            return new_data
        
        args = [str(random.random()) for _ in range(random.randint(0, 100))]
        kwargs = dict([
            (str(time.time()), sha.sha(str(random.random())).hexdigest())
            for _ in range(random.randint(0, 100))
        ])
        
        # store
        data0 = cached_func0(*args, **kwargs)
        data1 = cached_func1(*args, **kwargs)
        
        self.assertEqual(
            cached_func0(*args, **kwargs),
            data0
        )
        
        self.assertEqual(
            cached_func1(*args, **kwargs),
            data1
        )
        
        time.sleep(
            TTL + 1
        )
    
        self.assertNotEqual(
            cached_func0(*args, **kwargs),
            data0
        )
        
        self.assertEqual(
            cached_func1(*args, **kwargs),
            data1
        )
        
        data0 = cached_func0(*args, **kwargs)
        data1 = cached_func1(*args, **kwargs)
        
        self.engine.clear()
        self.eternal.clear()

        self.assertNotEqual(
            cached_func0(*args, **kwargs),
            data0
        )
        
        self.assertNotEqual(
            cached_func1(*args, **kwargs),
            data1
        )
        

#===============================================================================
# MEMCACHED CACHE TEST
#===============================================================================

class MemCachedCacheTest(BaseCacheTest, unittest.TestCase):
    
    def setUp(self):
        self.host = "localhost:11211"
        self.engine = memcached_cache.MemcachedCache((self.host,), debug=0, ttl=TTL)
        self.eternal = memcached_cache.MemcachedCache((self.host,), debug=0, ttl=None)
        if not self.engine.ping():
            self.skip_reason = "Unreacheable Memcached Server '{}'".format(self.host)
        
    def test_ping(self):
        engine = memcached_cache.MemcachedCache(("localhost:80",),
                                                  debug=0,
                                                  ttl=TTL)
        self.assertFalse(engine.ping())
        
  
#===============================================================================
# DICT_CACHE TEST
#===============================================================================


class DictCacheTest(BaseCacheTest, unittest.TestCase):
    """ Class doc """
    
    def setUp(self):
        self.engine = dict_cache.DictCache(ttl=TTL)
        self.eternal = dict_cache.DictCache(ttl=None)


#===============================================================================
# SQLITE_CACHE TEST
#===============================================================================

"""
@mutter
class SqliteCacheTest(BaseCacheTest):
    
    def setUp(self):
        tablename = str(random.random())
        db = tempfile.mktemp(suffix=".db")
        self.engine = sqlite_cache.SqliteCache(database=db,
                                               tablename=tablename,
                                               force_delete=True,
                                               ttl=TTL)
    def tearDown(self):
        self.engine.close()
        
    def test_duplicated_table(self):
        self.engine.close()
        self.assertRaises(ValueError, sqlite_cache.SqliteCache,
                          database=self.engine.database,
                          tablename=self.engine.tablename,
                          force_delete=False,
                          ttl=TTL)
"""     
    

#===============================================================================
# MAIN
#===============================================================================

if __name__ == "__main__":
    print(__doc__)
