#!/usr/bin/env python
# -*- coding: utf-8 -*-

# "THE WISKEY-WARE LICENSE":
# <jbc.develop@gmail.com> wrote this file. As long as you retain this notice you
# can do whatever you want with this stuff. If we meet some day, and you think
# this stuff is worth it, you can buy me a WISKEY in return Juan BC

#===============================================================================
# DOC
#===============================================================================

"""Test for infopython.interpolators

"""

#===============================================================================
# IMPORTS
#===============================================================================

import unittest
import random

from infopython import interpolator


#===============================================================================
# TEST
#===============================================================================

class BaseInterpolateTests(object):
    
    def test_interpolate_linear_set(self):
        
        data = set()
        for xy in range(1, 1000, 2):
            data.add((xy, xy))
        
        interp = self.interpolator(data)
        
        # test exact
        for x, y in data:
            self.assertAlmostEqual(interp(x), y, places=2)
        
        # test intermediate
        data_dict = dict(data)
        for x in range(10, 900, 2):
            interp_value = interp(x)
            self.assertTrue(
                interp_value > data_dict[x - 1] 
                and interp_value < data_dict[x + 1]
            )
            

    
#===============================================================================
# PieceWisePolynomial TEST
#===============================================================================


class PieceWisePolynomialTest(BaseInterpolateTests, unittest.TestCase):
    
    def setUp(self):
        self.interpolator = interpolator.PieceWisePolynomial
        

#===============================================================================
# LinearLeastSquares TEST
#===============================================================================
  
class LinearLeastSquares(BaseInterpolateTests, unittest.TestCase):
    
    def setUp(self):
        self.interpolator = interpolator.LinearLeastSquares
          
    
#===============================================================================
# Expert TEST
#===============================================================================
  
class ExpertSquares(BaseInterpolateTests):
    
    def setUp(self):
        self.interpolator = interpolator.Expert
    
    # this is the only interpolator wit no linear generic test
    def test_interpolate_linear_set(self):
        pass
    
    def test_interpolate_random_set(self):
        
        data = set()
        for xy in range(1, 1000, 2):
            data.add(
                (xy, random.random() + random.randint(0, 1000))
            )
        
        interp = self.interpolator(data)
        
        # test exact
        for x, y in data:
            self.assertEqual(interp(x), y)
        
        # test intermediate
        for x in range(10, 900, 2): 
            self.assertRaises(KeyError, interp, x)
            
    def test_get(self):
        data = {}
        for x in range(1, 1000, 2):
            data[x] = random.random() + random.randint(0, 1000)
        
        interp = self.interpolator(data.items())
        
        for x, y in data.items():
            d = random.random()
            yinterp = interp.get(x, d)
            self.assertEqual(yinterp, y)
            self.assertNotEqual(yinterp, d)
            
        for x in range(0, 1000, 2):
            d = random.random()
            self.assertEqual(interp.get(x, d), d)

    
#===============================================================================
# MAIN
#===============================================================================

if __name__ == "__main__":
    print(__doc__)
