#!/usr/bin/env python
# -*- coding: utf-8 -*-

# "THE WISKEY-WARE LICENSE":
# <jbc.develop@gmail.com> wrote this file. As long as you retain this notice you
# can do whatever you want with this stuff. If we meet some day, and you think
# this stuff is worth it, you can buy me a WISKEY in return Juan BC

#===============================================================================
# DOC
#===============================================================================

"""Test for infopython.media

"""

#===============================================================================
# IMPORTS
#===============================================================================

import unittest
import random
import hashlib

from infopython import media

#===============================================================================
# TEST
#===============================================================================

class MediaTests(unittest.TestCase):
    
    
    def setUp(self):
        name0 = hashlib.sha1(
            str(random.random())
        ).hexdigest()
        name1 = hashlib.sha1(
            str(random.random())
        ).hexdigest()
        while name1 == name0:
            name1 = hashlib.sha1(
                str(random.random())
            ).hexdigest()
        self.medias = media.Media(name0), media.Media(name1) 
    
    def test_eq(self):
        for m0 in self.medias:
            for m1 in self.medias:
                  
                m0c = media.Media(m0.name)
                m1c = media.Media(m1.name)
        
                self.assertEqual(m0, m0c)
                self.assertEqual(m0.name, m0c.name)
                self.assertFalse(m0 is m0c)
                
                self.assertEqual(m1, m1c)
                self.assertEqual(m1.name, m1c.name)
                self.assertFalse(m1 is m1c)
        
                if m0 is m1:
                    self.assertEqual(m0, m1)
                    self.assertEqual(m1, m0)
                    self.assertEqual(m1.name, m0.name)
                else:
                    self.assertNotEqual(m0, m1)
                    self.assertNotEqual(m1, m0)
                    self.assertNotEqual(m1.name, m0.name)
                    
            
    def test_hash(self):       

        for m0 in self.medias:
            for m1 in self.medias:
                m0c, m1c = media.Media(m0.name), media.Media(m1.name)
                
                self.assertEqual(hash(m0), hash(m0c))
                self.assertFalse(m1 is m0c)
                self.assertEqual(hash(m1), hash(m1c))
                self.assertFalse(m1 is m1c)
                if m0 is m1:
                    self.assertEqual(hash(m1), hash(m0))
                else:
                    self.assertNotEqual(hash(m1), hash(m0))
                    
                s = set()
                s.add(m0)
                self.assertEqual(len(s), 1)
                self.assertIn(m0, s)
                self.assertIn(m0c, s)
                if m0 is m1:
                    self.assertIn(m1, s)
                    self.assertIn(m1c, s)
                else:
                    self.assertNotIn(m1, s)
                    self.assertNotIn(m1c, s)
                s.add(m1)
                if m0 is m1:
                    self.assertEqual(len(s), 1)
                else:
                    self.assertEqual(len(s), 2)
                self.assertIn(m0, s)
                self.assertIn(m0c, s)
                self.assertIn(m1, s)
                self.assertIn(m1c, s)
                
                
    def test_repr(self):
        for m in self.medias:
            expected = "<{classname} (name={name}) at {id}>".format(
                classname=type(m).__name__,
                name=str(m.name),
                id=hex(id(m))
            )
            self.assertEqual(repr(m), expected)
              
    def test_str(self):
        for m in self.medias:
            expected = "<{classname} (name={name}) at {id}>".format(
                classname=type(m).__name__,
                name=str(m.name),
                id=hex(id(m))
            )
            self.assertEqual(str(m), expected)
               
    
#===============================================================================
# MAIN
#===============================================================================

if __name__ == "__main__":
    print(__doc__)

