#!/usr/bin/env python
# -*- coding: utf-8 -*-

# "THE WISKEY-WARE LICENSE":
# <jbc.develop@gmail.com> wrote this file. As long as you retain this notice you
# can do whatever you want with this stuff. If we meet some day, and you think
# this stuff is worth it, you can buy me a WISKEY in return Juan BC

#===============================================================================
# DOC
#===============================================================================

"""Test for infopython.util.frozendict

"""

#===============================================================================
# IMPORTS
#===============================================================================

import unittest
import random
import sha

from infopython.util import frozendict


#===============================================================================
# TEST
#===============================================================================

class FrozenDictTest(unittest.TestCase):
    
    def setUp(self):
        self.rd = {}
        for i in range(random.randint(1, 100)):
            self.rd[str(i)] = sha.sha(str(random.random())).hexdigest()
    
    def test_creation(self):
        frozendict.FrozenDict(self.rd)
        frozendict.FrozenDict(**self.rd)
        frozendict.FrozenDict(self.rd.items())
        frozendict.FrozenDict(frozendict.FrozenDict(self.rd.items()))
        self.assertRaises(ValueError, frozendict.FrozenDict, self.rd.values())
        
    
    def test_clear(self):
        fd = frozendict.FrozenDict(self.rd)
        self.assertEqual(len(fd), len(self.rd))
        self.assertRaises(AttributeError, fd.clear)
        
    def test_copy(self):
        fd = frozendict.FrozenDict(self.rd)
        fd_copy = fd.copy()
        self.assertTrue(isinstance(fd_copy, frozendict.FrozenDict))
        self.assertTrue(fd == fd_copy)
        self.assertTrue(fd is not fd_copy)
        
    def test_fromkeys(self):
        v = str(random.random())
        fd = frozendict.FrozenDict.fromkeys(self.rd.keys(), v)
        for k in self.rd.keys():
            self.assertEqual(fd[k], v)
        
    def test_get(self):
        fd = frozendict.FrozenDict(self.rd)
        for k in self.rd.keys():
            self.assertEqual(fd.get(k, None), self.rd[k])
        fake_key = str(random.random())
        while fake_key in fd.keys():
            fake_key += str(random.random)
        fake_value = str(random.random())
        while fake_key in fd.values():
            fake_value += str(random.random)
        self.assertEqual(fd.get(fake_key, fake_value), fake_value)
            
    def test_has_key_or_in(self):
        fd = frozendict.FrozenDict(self.rd)
        for k in self.rd.keys():
            self.assertTrue(k in fd)
            self.assertTrue(fd.has_key(k))
        fake_key = str(random.random())
        while fake_key in fd.keys():
            fake_key += str(random.random)
        self.assertTrue(fake_key not in fd)
        self.assertTrue(not fd.has_key(fake_key))
        
    def test_items(self):
        fditems = frozendict.FrozenDict(self.rd).items()
        for item in self.rd.items():
            self.assertTrue(item in fditems)
        fditems = frozendict.FrozenDict(self.rd).viewitems()
        for k in self.rd.items():
            self.assertTrue(k in fditems)
            
    def test_keys(self):
        fdkeys = frozendict.FrozenDict(self.rd).keys()
        for key in self.rd.keys():
            self.assertTrue(key in fdkeys)
        fdkeys = frozendict.FrozenDict(self.rd).viewkeys()
        for k in self.rd.keys():
            self.assertTrue(k in fdkeys)
        
    def test_values(self):
        fdvalues = frozendict.FrozenDict(self.rd).values()
        for value in self.rd.values():
            self.assertTrue(value in fdvalues)
        fdvalues = frozendict.FrozenDict(self.rd).viewvalues()
        for k in self.rd.values():
            self.assertTrue(k in fdvalues)

    def test_pop(self):
        fd = frozendict.FrozenDict(self.rd)
        for k in fd.keys():
            self.assertRaises(AttributeError, fd.pop, k)
            self.assertRaises(AttributeError, fd.pop, k, str(random.random()))
            
    def test_popitem(self):
        fd = frozendict.FrozenDict(self.rd)
        for i in range(len(fd)):
            self.assertRaises(AttributeError, fd.popitem)
            
    def test_setdefault(self):
        fd = frozendict.FrozenDict(self.rd)
        for k in fd.keys():
            self.assertRaises(
                AttributeError, fd.setdefault, k, str(random.random())
            )
 
    def test_update(self):
        fd = frozendict.FrozenDict()
        self.assertRaises(AttributeError, fd.update, self.rd)
        
    def test_del(self):
        fd = frozendict.FrozenDict(self.rd)
        for k in fd.keys():
            try:
                del fd[k]
            except AttributeError:
                pass
            else:
                self.fail("deleted item from a frozendict")
 
    def test_setitem(self):
        fd = frozendict.FrozenDict(self.rd)
        for k in fd.keys():
            try:
                fd[k] = random.random()
            except AttributeError:
                pass
            else:
                self.fail("setted item from a frozendict")
                
    def test_getitem(self):
        fd = frozendict.FrozenDict(self.rd)
        for k in fd.keys():
            fd[k]
  
    
#===============================================================================
# MAIN
#===============================================================================

if __name__ == "__main__":
    print(__doc__)
